//
// BeInternetCentral
// Window class
//
// (c) 1997 Hubert Figuiere
// figuiere@kagi.com
//
// 26june1997
//
// version 0.6 - 24July1997
//	-added workaround BeOS PR Bug #970713-190126
//	-added basic Personality management


#include "app/BICApplication.h"
#include "app/BICWindow.h"
#include "app/BICCommands.h"
#include "app/TopicListItem.h"
#include "app/PersonalityRequester.h"
#include "sdk/BICKeyNames.h"


#define PERSONALITY_TOPIC_IDX	0
#define EMAIL_TOPIC_IDX			1
#define NEWS_TOPIC_IDX			2
#define HELPERS_TOPIC_IDX		3



///////////////////////////////////////////////////////////////////
BICWindow::BICWindow(const BPoint & pos)
	: BWindow(BRect(0,0,500,300), "BeInternetCentral", B_TITLED_WINDOW,
				B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BView * background;
	mCurrentTopicView = NULL;
	BRect aRect;
	BRect winBounds(this->Bounds());
	MoveTo(pos);
	
	CreateMenuBar(winBounds);
	winBounds.top += 18.0;
	/*insert code to show view here*/
	background = new BView(winBounds, "background", B_FOLLOW_ALL, B_WILL_DRAW);
	AddChild(background);
	winBounds = background->Bounds();
	background->SetViewColor(238, 238, 238, 0);
	
	aRect.Set(10, 10, 110, winBounds.bottom - 10);
	mTopics = new BListView(aRect, "topics", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL, 
				B_NAVIGABLE | B_WILL_DRAW);
	background->AddChild(mTopics);
	mTopics->SetSelectionMessage(BICWindow::MakeCommandMessage(TOPIC_SELECTED));
	mTopics->SetTarget(this);
	FillTopicList(mTopics);
	
	aRect.Set(aRect.right + 10.0, aRect.top, winBounds.right - 10, aRect.bottom); 

	/*personality view*/ 
	mPersonalityView = new PersonalityView(aRect);
	background->AddChild(mPersonalityView);
	mPersonalityView->Hide();
	/*email view*/ 
	mEmailView = new EmailView(aRect);
	background->AddChild(mEmailView);
	mEmailView->Hide();
	/*news view*/ 
	mNewsView = new NewsView(aRect);
	background->AddChild(mNewsView);
	mNewsView->Hide();
	/*helpers view*/ 
	mHelpersView = new HelpersView(aRect);
	background->AddChild(mHelpersView);
	mHelpersView->Hide();
	
	mPersonalityRequester = new PersonalityRequester(this);
	mPersonalityRequester->Show();
	mPersonalityRequester->Hide();
	
	mTopics->Select(PERSONALITY_TOPIC_IDX);
	
	BMessage * aMsg = new BMessage(SET_PERSONALITY_CMD_MSG);
	aMsg->AddString(PERSONALITY_LABEL, gBicRegistry->GetDefaultPersonality());
	
	PostMessage(aMsg);
	
	delete aMsg;
}


///////////////////////////////////////////////////////////////////
BICWindow::~BICWindow()
{
	if (mPersonalityRequester != NULL) {
		if (mPersonalityRequester->Lock()) {
			mPersonalityRequester->Quit();
			mPersonalityRequester = NULL;
		}
	}
}


///////////////////////////////////////////////////////////////////
bool BICWindow::QuitRequested()
{
	/* inserted confirm and save code here...*/
	bool doExit = true;
	int32 mr;
	if (mDirty) {
		BAlert * alert = new BAlert("save?", "Save settings before quitting ?",
					"Don't Save", "Cancel","Save", B_WIDTH_AS_USUAL, B_WARNING_ALERT);
		mr = alert->Go();
		switch (mr) {
			case 0:
				//don't save
				break;
			case 1:
				doExit = false;
				break;
			case 2:
				//save
				Save();
				break;
		}
	}
	
	if (doExit) {
		be_app->PostMessage(B_QUIT_REQUESTED);	
	}
	return doExit;
}



///////////////////////////////////////////////////////////////////
void BICWindow::DispatchMessage(BMessage * msg, BHandler * handler)
{
	switch (msg->what) {
		case STANDARD_CMD_MSG:
			ExecuteCommand(msg);
			break;
		case SET_PERSONALITY_CMD_MSG:
			printf("SET_PERSONALITY_CMD_MSG received\n");
			mPersonalityView->MessageReceived(msg);
			mEmailView->MessageReceived(msg);
			mNewsView->MessageReceived(msg);
			mHelpersView->MessageReceived(msg);
			break;
		default:
			inherited::DispatchMessage(msg, handler);
	}
}


#pragma mark -


///////////////////////////////////////////////////////////////////
void BICWindow::CreateMenuBar(const BRect & bounds)
{
	BMenuItem * anItem;
	BMenu * aMenu;
	BMenuBar * aMenuBar = new BMenuBar(bounds, "mainMenu");
	AddChild(aMenuBar);
	
	aMenu = new BMenu("File");
	aMenuBar->AddItem(aMenu);
		anItem = new BMenuItem("Open Standard Settings", 
					BICWindow::MakeCommandMessage(OPEN_STANDARD_SETTINGS));
		aMenu->AddItem(anItem);

		anItem = new BMenuItem("Open…", BICWindow::MakeCommandMessage(OPEN_SETTINGS), 'O');
		aMenu->AddItem(anItem);

		anItem = new BMenuItem("Save", BICWindow::MakeCommandMessage(SAVE_SETTINGS), 'S');
		aMenu->AddItem(anItem);

		anItem = new BMenuItem("Save As…", BICWindow::MakeCommandMessage(SAVE_SETTINGS_AS));
		aMenu->AddItem(anItem);

		aMenu->AddItem(new BSeparatorItem);

		anItem = new BMenuItem("About", new BMessage(B_ABOUT_REQUESTED));
		anItem->SetTarget(be_app);
		aMenu->AddItem(anItem);

		aMenu->AddItem(new BSeparatorItem);

		anItem = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
		aMenu->AddItem(anItem);

	aMenu = new BMenu("Edit");
	aMenuBar->AddItem(aMenu);
		anItem = new BMenuItem("Undo", BICWindow::MakeCommandMessage(UNDO_COMMAND), 'Z');
		aMenu->AddItem(anItem);

		aMenu->AddItem(new BSeparatorItem);

		anItem = new BMenuItem("Cut", BICWindow::MakeCommandMessage(CUT_COMMAND), 'X');
		aMenu->AddItem(anItem);
		
		anItem = new BMenuItem("Copy", BICWindow::MakeCommandMessage(COPY_COMMAND), 'C');
		aMenu->AddItem(anItem);
		
		anItem = new BMenuItem("Paste", BICWindow::MakeCommandMessage(PASTE_COMMAND), 'V');
		aMenu->AddItem(anItem);
		
		anItem = new BMenuItem("Delete", BICWindow::MakeCommandMessage(DELETE_COMMAND));
		aMenu->AddItem(anItem);
	
	aMenu = new BMenu("Settings");
	aMenuBar->AddItem(aMenu);
	aMenu->SetRadioMode(true);
		mPersonalityMItem = new BMenuItem("Personality", BICWindow::MakeCommandMessage(PERSONALITY_TOPIC), '1');
		aMenu->AddItem(mPersonalityMItem);

		mEmailMItem = new BMenuItem("E-mail", BICWindow::MakeCommandMessage(EMAIL_TOPIC), '2');
		aMenu->AddItem(mEmailMItem);
		
		mNewsMItem = new BMenuItem("News", BICWindow::MakeCommandMessage(NEWS_TOPIC), '3');
		aMenu->AddItem(mNewsMItem);

		mHelperMItem = new BMenuItem("Helpers", BICWindow::MakeCommandMessage(HELPERS_TOPIC), '4');
		aMenu->AddItem(mHelperMItem);
}




///////////////////////////////////////////////////////////////////
void BICWindow::FillTopicList(BListView * topics)
{
	TopicListItem * aTopic;

	aTopic = new TopicListItem;
	aTopic->SetTopic("Personality");
	topics->AddItem(aTopic);

	aTopic = new TopicListItem;
	aTopic->SetTopic("E-mail");
	topics->AddItem(aTopic);

	aTopic = new TopicListItem;
	aTopic->SetTopic("News");
	topics->AddItem(aTopic);

	aTopic = new TopicListItem;
	aTopic->SetTopic("Helpers");
	topics->AddItem(aTopic);
}



///////////////////////////////////////////////////////////////////
void BICWindow::ExecuteCommand(BMessage * msg)
{
	status_t err;
	int32 command = NO_COMMAND;
	err = msg->FindInt32(CMD_IDX_LABEL, &command);
	if (err == B_NO_ERROR) {
		switch(command) {
			case SAVE_SETTINGS:
				// workaround because of a BeOS bugs:
				// if you select a menu, the current control loose its focus before 
				//   having it again
				// if you use a keyboard, it doesn't loose it's focus and thus the message
				//  is NOT sent...
				{
					BView * aView = CurrentFocus();
					if (aView != NULL) {
						mTopics->MakeFocus();
						aView->MakeFocus();
					}
					// after that, the configuration will need to be saved again... :-(
				}
				//
				Save();
				break;
			case TOPIC_SELECTED: {
					int32 idx;
					err = msg->FindInt32("index", &idx);
					if (err == B_NO_ERROR) {
						SelectTopic(idx);
					}
				}
				break;
			case PERSONALITY_TOPIC:
				SelectTopic(PERSONALITY_TOPIC_IDX);
				break;
			case EMAIL_TOPIC:
				SelectTopic(EMAIL_TOPIC_IDX);
				break;
			case NEWS_TOPIC:
				SelectTopic(NEWS_TOPIC_IDX);
				break;
			case HELPERS_TOPIC:
				SelectTopic(HELPERS_TOPIC_IDX);
				break;
			case NNTPSERVER_CHANGED:
				SaveTextField(msg, KEY_NNTP_SERVER, "");
				break;
			case NNTPLOGIN_CHANGED:
				SaveTextField(msg, KEY_NNTP_LOGIN, "");
				break;
			case NNTPPASSWORD_CHANGED:
				SaveTextField(msg, KEY_NNTP_PASSWORD, "");
				break;
			case SMTPRELAY_CHANGED:
				SaveTextField(msg, KEY_SMTP_HOST, mCurrentTopicView->CurrentPersonality());
				break;
			case POPACOUNT_CHANGED:
				SaveTextField(msg, KEY_POP_USERNAME, mCurrentTopicView->CurrentPersonality());
				break;
			case POPHOST_CHANGED:
				SaveTextField(msg, KEY_POP_HOST, mCurrentTopicView->CurrentPersonality());
				break;
			case POPPASSWORD_CHANGED:
				SaveTextField(msg, KEY_POP_PASSWORD, mCurrentTopicView->CurrentPersonality());
				break;
			case NAME_CHANGED:
				SaveTextField(msg, KEY_REAL_NAME, mCurrentTopicView->CurrentPersonality());
				break;
			case EMAIL_CHANGED:
				SaveTextField(msg, KEY_EMAIL_ADRESS, mCurrentTopicView->CurrentPersonality());
				break;
			case ORGANIZATION_CHANGED:
				SaveTextField(msg, KEY_ORGANIZATION, mCurrentTopicView->CurrentPersonality());
				break;
			case PERSONALITY_CHANGED:
				PersonalityChanged(msg);
				printf("PERSONALITY_CHANGED received\n");
				break;
			case ADD_PERSONALITY:
				mPersonalityRequester->Show();
				break;
			case VALIDATE_REQUESTER:
				mPersonalityRequester->Hide();
				PersonalityAdded(mPersonalityRequester->GetValue());
			default:
				break;
		}
	}
}



///////////////////////////////////////////////////////////////////
void BICWindow::SelectTopic(int32 idx)
{
	if (Lock()) {
		if (mCurrentTopicView != NULL) {
			mCurrentTopicView->Hide();
			mCurrentTopicView = NULL;
		}
		mTopics->Select(idx);
		switch(idx) {
			case PERSONALITY_TOPIC_IDX:
				mCurrentTopicView = mPersonalityView;
				mPersonalityMItem->SetMarked(true);
				break;
			case EMAIL_TOPIC_IDX:
				mCurrentTopicView = mEmailView;
				mEmailMItem->SetMarked(true);
				break;
			case NEWS_TOPIC_IDX:
				mCurrentTopicView = mNewsView;
				mNewsMItem->SetMarked(true);
				break;
			case HELPERS_TOPIC_IDX:
				mCurrentTopicView = mHelpersView;
				mHelperMItem->SetMarked(true);				
				break;
		}
		mCurrentTopicView->Show();
		Unlock();
	}
}


///////////////////////////////////////////////////////////////////
void BICWindow::Save()
{
	status_t err;

	err = gBicRegistry->SaveRegistry();
	if (err == B_NO_ERROR) {
		mDirty = false;
	}
	else {
		printf("error while saving : %lx\n", err);
	}
}

#pragma mark -

///////////////////////////////////////////////////////////////////
/*
	Create standard command message
*/
BMessage * BICWindow::MakeCommandMessage(int32 cmdIndex)
{
	BMessage * aMsg = new BMessage(STANDARD_CMD_MSG);
	aMsg->AddInt32(CMD_IDX_LABEL, cmdIndex);
	
	return aMsg;	
}


///////////////////////////////////////////////////////////////////
void BICWindow::SaveTextField(const BMessage * msg, const key_id id, const key_attribute_id attribute)
{
	BTextControl * field;
	status_t err;
	
	err = msg->FindPointer("source", &field);
	
	if (err == B_NO_ERROR) {
		printf("saving TextField : %s %s\n", id, attribute);
		gBicRegistry->SetKey(id, attribute, field->Text());
		mDirty = true;
	}
}

///////////////////////////////////////////////////////////////////
void BICWindow::PersonalityChanged(BMessage * msg)
{
	BMenuItem * menuItem;
	status_t err;
	BMessage * aMsg;
	
	printf("BICWindow::PersonalityChanged\n");
	
	err = msg->FindPointer("source", &menuItem);
	if (err != B_NO_ERROR) {
		printf("error FindPointer\n");
		return;
	}
	
	aMsg = new BMessage(SET_PERSONALITY_CMD_MSG);
	aMsg->AddString(PERSONALITY_LABEL, menuItem->Label());
	PostMessage(aMsg);
	delete aMsg;
}



///////////////////////////////////////////////////////////////////
void BICWindow::PersonalityAdded(const char * personality)
{
	status_t err;
	
	if (personality[0] == 0) {
		printf("no personality\n");
		return;
	}

	err = gBicRegistry->AddPersonality(personality);
	
	if (err == B_NO_ERROR) {		
		mPersonalityView->NeedReloadPersonalityList(personality);
		mEmailView->NeedReloadPersonalityList(personality);
		mNewsView->NeedReloadPersonalityList(personality);
		mHelpersView->NeedReloadPersonalityList(personality);
		mDirty = true;
	}
	else {
		//report the error...
		printf("error gBicRegistry->AddPersonality : %ld\n", err);
	}
}


///////////////////////////////////////////////////////////////////
