//
// BeInternetCentral
// TopicView class
//
// (c) 1997 Hubert Figuiere
// figuiere@kagi.com
//
// 30june1997


#include "app/BICApplication.h"
#include "app/BICCommands.h"
#include "app/TopicView.h"
#include "sdk/libbic.h"



///////////////////////////////////////////////////////////////
TopicView::TopicView(BRect frame, const char * name)
	: BView(frame, "personalityView", B_FOLLOW_ALL, B_WILL_DRAW)
{
	if (name != NULL) {
		mName = strdup(name);
	}	
	mPersonalityMenu = NULL;
	mCurrentPersonality = (key_attribute_id)malloc((KEY_NAME_LENGTH + 1) * sizeof(char));
	strcpy(mCurrentPersonality, gBicRegistry->GetDefaultPersonality());
}



///////////////////////////////////////////////////////////////
TopicView::~TopicView()
{
	if (mName != NULL) {
		free(mName);
	}
	if (mCurrentPersonality != NULL) {
		free(mCurrentPersonality);
	}
}


///////////////////////////////////////////////////////////////
void TopicView::AttachedToWindow(void)
{
	inherited::AttachedToWindow();
	SetViewColor(Parent()->ViewColor());
	mBox = new BBox(Bounds());
	AddChild(mBox);
	mBox->SetLabel(mName);
}


///////////////////////////////////////////////////////////////
void TopicView::MessageReceived(BMessage * msg)
{
	status_t err;
	char * str;
	
	switch (msg->what) {
		case SET_PERSONALITY_CMD_MSG:
			err = msg->FindString(PERSONALITY_LABEL, &str);
			if (err == B_NO_ERROR) {
				strcpy(mCurrentPersonality, str);
				//printf("changing current personality : %s\n", mCurrentPersonality);
				CurrentPersonalityChanged(msg);
			}
			break;
		default:
			inherited::MessageReceived(msg);
	}
}



///////////////////////////////////////////////////////////////
void TopicView::ReloadPersonalityList(BPopUpMenu * personalityMenu, const char * currentPersonality)
{
	ssize_t attrSize;
	status_t err;
	BMenuItem * anItem;
	char personality [KEY_NAME_LENGTH + 1];
	int32 i, count;
	BMessage * aMsg;

	if (personalityMenu == NULL) {
		return;
	}

	/*personalities... should be optimized a bit...*/
	count = personalityMenu->CountItems();
	for (i = 0; i < count; i++) {
		anItem = personalityMenu->RemoveItem((int32)0);	//if I don't cast, CW generates an error
		if (anItem == NULL) {
			break;
		}
		delete anItem;
	}

	err = gBicRegistry->CountPersonality(count);
	if (err == B_NO_ERROR) {
		for (i = 0; i < count; i++) {
			attrSize = KEY_NAME_LENGTH;
			err = gBicRegistry->GetPersonality(i, personality, attrSize);
			if (err == B_NO_ERROR) {
				aMsg = BICWindow::MakeCommandMessage(PERSONALITY_CHANGED);
				anItem = new BMenuItem(personality, aMsg);
				personalityMenu->AddItem(anItem);
				anItem->SetTarget(Window());
				if (strcmp(personality, currentPersonality) == 0) {
					anItem->SetMarked(true);
					// hopw can I invoke a BMenuItem ?
					BMessage msgClone(anItem->Message());
					msgClone.AddPointer("source", anItem);
					msgClone.AddInt32("index", anItem->Menu()->IndexOf(anItem));
					msgClone.AddInt64("when", system_time());
					((BWindow*)(anItem->Target()))->PostMessage(&msgClone);
				}
			}
		}
	}
}


///////////////////////////////////////////////////////////////
status_t TopicView::LoadKeyIntoTextControl(BTextControl * textControl, 
									const key_id theKey, const key_attribute_id personality,
									key_value & value, ssize_t & size)
{
	status_t err;
	ssize_t realSize;

	realSize = size;
	err = gBicRegistry->GetKey(theKey, personality, value, realSize);
	if (err == BIC_BUFFER_TOO_SMALL) {
		value = (key_value)realloc(value, (realSize + 1) * sizeof(char));
		size = realSize;
		err = gBicRegistry->GetKey(theKey, personality, value, realSize);
	}	
	if (err == B_NO_ERROR) {
		//set up the value...
		textControl->SetText(value);
	}
	else {
		textControl->SetText("");
	}
	
	return err;
}


///////////////////////////////////////////////////////////////
