//
//
// BeInternetCentral Lib
//
// ©1997 Hubert Figuière
//
// May 26th 1997

#ifndef _LIBBIC_H_
#define _LIBBIC_H_

#include "sdk/BICErrors.h"

typedef char * key_id;			/* key from the registry */
typedef char * key_value;		/* a key value */
typedef char * key_attribute_id;

#define KEY_ATTRIBUTE_DEFAULT_ID	0

#define KEY_NAME_LENGTH		128



typedef uint32 bic_version;		/* bic version */

#define BIC_CURRENT_VERSION		5

/* internal class for the BIC Registry */
class BICInternals;

class BICRegistry
{
	friend status_t create_bic_registry(BICRegistry * &, bic_version & vers);
	friend status_t dispose_bic_registry(BICRegistry * &);
	
	public:
virtual status_t InitCheck(void);
virtual status_t GetKey(const key_id id, const key_attribute_id attribute, key_value value, ssize_t & size);
virtual status_t SetKey(const key_id id, const key_attribute_id attribute, const key_value value);
virtual status_t DeleteKey(const key_id id, const key_attribute_id attribute);
		/*personality access*/
virtual status_t CountPersonality(int32 & count);
virtual char * GetDefaultPersonality() const;
virtual status_t SetDefaultPersonality(const key_attribute_id personality);
virtual status_t GetPersonality(const int32 index, key_attribute_id personality, ssize_t & size) const;
virtual status_t AddPersonality(const key_attribute_id personality);
virtual status_t DeletePersonality(const key_attribute_id personality);
		/*registry access... might be changed in future implementations */
virtual status_t SetRegPath(const char * dir, const char * name);
virtual status_t GetRegPath(const char * dir, char * name);
virtual status_t ResetRegPath(void);
virtual status_t SaveRegistry(void);
		/*multi-thread access...*/
virtual bool Lock();
virtual void Unlock();
virtual bool IsLocked();
				
	private:
		BICRegistry();
		virtual ~BICRegistry();
		BICInternals * 	_private_datas_;
};


#include "sdk/BICKeyNames.h"

/* these functions will be implemented in bicglue.a */

#pragma export on

extern "C" {

	extern status_t create_bic_registry(BICRegistry * &, bic_version & vers);
	extern status_t dispose_bic_registry(BICRegistry * &);

}

#pragma export reset


#endif
